import requests

from datastation.common.utils import print_dry_run_message


class DataverseApi:
    def __init__(self, server_url, api_token):
        self.server_url = server_url
        self.api_token = api_token

    def search(self, subtree="root", query="*", dry_run=False):
        """
        Do a query via the public search API, only published datasets
        using the public search 'API', so no token needed

        Note that the current functionality of this function is very limited!

        :param subtree: This is the collection (dataverse alias)
                        it recurses into a collection and its children etc. very useful with nesting collection
        :param start: The cursor (zero based result index) indicating where the result page starts
        :param rows: The number of results returned in the 'page'
        :return: The 'paged' search results in a list of dictionaries
        """

        per_page = 25
        current_page = 0

        # always type=dataset, those have pids (disregarding pids for files)
        params = {
            "q": query,
            "subtree": subtree,
            "type": "dataset",
            "per_page": str(per_page),
            "start": str(current_page),
        }

        headers = {"X-Dataverse-key": self.api_token}
        url = f"{self.server_url}/api/search"

        if dry_run:
            print_dry_run_message(method="GET", url=url, headers=headers, params=params)
            return None

        while True:
            dv_resp = requests.get(url, headers=headers, params=params)
            dv_resp.raise_for_status()

            data = dv_resp.json()["data"]
            items = data["items"]

            if len(items) == 0:
                break

            for item in items:
                yield item

            current_page += per_page
            params["start"] = str(current_page)

    def get_contents(self, dry_run=False):
        headers = {"X-Dataverse-key": self.api_token}
        url = f"{self.server_url}/api/dataverses/root/contents"

        if dry_run:
            print_dry_run_message(method="GET", url=url, headers=headers)
            return None

        dv_resp = requests.get(url, headers=headers)
        dv_resp.raise_for_status()

        resp_data = dv_resp.json()["data"]
        return resp_data



import logging
import os
import time

from datastation.common.csv import CsvReport


def get_pids(pid_or_file):
    if os.path.isfile(os.path.expanduser(pid_or_file)):
        pids = []
        with open(os.path.expanduser(pid_or_file)) as f:
            for line in f:
                pids.append(line.strip())
        return pids
    else:
        return [pid_or_file]


class BatchProcessor:
    def __init__(self, wait=0.1, fail_on_first_error=True):
        self.wait = wait
        self.fail_on_first_error = fail_on_first_error

    def process_pids(self, pids, callback):
        num_pids = len(pids)
        logging.info(f"Start batch processing on {num_pids} pids")
        i = 0
        for pid in pids:
            i += 1
            try:
                logging.info(f"Processing {i} of {num_pids}: {pid}")
                callback(pid)
                if self.wait > 0 and i < num_pids:
                    logging.debug(f"Waiting {self.wait} seconds before processing next pid")
                    time.sleep(self.wait)
            except Exception as e:
                logging.exception("Exception occurred", exc_info=True)
                if self.fail_on_first_error:
                    logging.error(f"Stop processing because of an exception: {e}")
                    break
                logging.debug("fail_on_first_error is False, continuing...")


class BatchProcessorWithReport(BatchProcessor):

    def __init__(self, report_file=None, headers=None, wait=0.1, fail_on_first_error=True):
        super().__init__(wait, fail_on_first_error)
        if headers is None:
            headers = ["DOI", "Modified", "Change"]
        self.report_file = report_file
        self.headers = headers

    def process_pids(self, pids, callback):
        with CsvReport(os.path.expanduser(self.report_file), self.headers) as csv_report:
            super().process_pids(pids, lambda pid: callback(pid, csv_report))



import argparse

from datastation.common.config import init
from datastation.common.utils import add_dry_run_arg, positive_int_argument_converter
from datastation.dataverse.dataverse_client import DataverseClient
from datastation.dataverse.notifications import Notifications


def main():
    config = init()
    parser = argparse.ArgumentParser(description="Clean up old notifications")
    parser.add_argument(
        "--days-old",
        help="Minimum amount of days old",
        type=positive_int_argument_converter,
        required=True,
    )
    add_dry_run_arg(parser)
    args = parser.parse_args()

    dataverse_client = DataverseClient(config['dataverse'])

    notifications = Notifications(
        dataverse_client, 
        dry_run=args.dry_run
    )
    
    notifications.cleanup(args.days_old)


if __name__ == "__main__":
    main()



import json
import logging
import os
import shutil

import requests

from datastation.common.utils import has_file_pred, has_dirtree_pred, is_sub_path_of, get_size, sizeof_fmt, \
    set_permissions


def is_deposit(path):
    return os.path.exists(os.path.join(path, 'deposit.properties'))


def is_file_writeable_to_group(f):
    return os.stat(f).st_mode & os.stat.S_IWGRP > 0


class IngestFlow:

    def __init__(self, config, dry_run: bool = False):
        self.service_baseurl = config['service_baseurl']
        self.ingest_areas = config['ingest_areas']
        self.dry_run = dry_run

    def set_dry_run(self, dry_run: bool):
        self.dry_run = dry_run

    def start_ingest(self, deposit_path, is_single_deposit, continue_previous, is_migration):
        if is_single_deposit:
            if not has_file_pred(deposit_path, is_deposit):
                print('ERROR: %s is not a deposit' % deposit_path)
                return
        else:
            if not has_dirtree_pred(deposit_path, is_deposit):
                print('ERROR: %s is not a batch of deposits' % deposit_path)
                return

        if not has_dirtree_pred(deposit_path, is_file_writeable_to_group):
            chmod_command = 'chmod -R g+w %s' % deposit_path
            print(f'Some files in the import batch do not give the owner group write permissions. '
                  f'Executing "{chmod_command}" to fix it')
            status = os.system(chmod_command)
            if status != 0:
                print('Could not give owner group write permissions. Possibly your account is not the owner user '
                      'of the files.')
                return

        command = {
            'inputPath': deposit_path,
            'batch': not is_single_deposit,
            'continue': continue_previous
        }
        if self.dry_run:
            logging.info("DRY-RUN: only printing command, not sending it...")
            print(json.dumps(command, indent=2))
        else:
            r = requests.post(f'{self.service_baseurl}/{"migrations" if is_migration else "imports"}/:start',
                              json=command)
            print(f'Server responded: {r.text}')

    def block_target(self, target):
        url = f'{self.service_baseurl}/blocked-targets/{target}'

        if self.dry_run:
            logging.info("DRY-RUN: only printing command, not sending it...")
            print(f'Request: POST {url}')
        else:
            r = requests.post(url)
            payload = r.json()

            if 'message' not in payload:
                print(f'Unexpected output format; expecting "message" property in result, not found: {payload}')

            if r.status_code != 200:
                print(f'ERROR: {r.status_code} - {payload["message"]}')
            else:
                print(f'Server responded: {r.status_code} - {payload["message"]}')

    def unblock_target(self, target):
        url = f'{self.service_baseurl}/blocked-targets/{target}'

        if self.dry_run:
            logging.info("DRY-RUN: only printing command, not sending it...")
            print(f'Request: DELETE {url}')
        else:
            r = requests.delete(url)
            payload = r.json()

            if 'message' not in payload:
                print(f'Unexpected output format; expecting "message" property in result, not found: {payload}')

            if r.status_code != 200:
                print(f'ERROR: {r.status_code} - {payload["message"]}')
            else:
                print(f'Server responded: {r.status_code} - {payload["message"]}')

    def list_events(self, source=None, deposit=None):
        url = f'{self.service_baseurl}/events'
        params = {}
        if deposit is not None:
            params = {'deposit': deposit}
        elif source is not None:
            params = {'source': source}

        if self.dry_run:
            logging.info("DRY-RUN: only printing command, not sending it...")
            print(f'Request: GET {url}')
        else:
            r = requests.get(url, params=params)
            print(r.text)

    def progress_report(self, batch_dir):
        abs_batch_dir = os.path.abspath(batch_dir)
        ingest_area = next(filter(lambda ia: is_sub_path_of(abs_batch_dir, ia['inbox']), self.ingest_areas), None)
        if ingest_area is None:
            print("ERROR: batch_dir {} does not seems to be in one of the inboxes: {}".format(batch_dir, ingest_area))
            return 1
        else:
            logging.debug("Found ingest_area: {}".format(ingest_area))
            rel_batch_dir = os.path.relpath(abs_batch_dir, ingest_area['inbox'])
            logging.debug("Relative batch dir: {}".format(rel_batch_dir))
            abs_out_dir = os.path.join(ingest_area['outbox'], rel_batch_dir)
            logging.debug("Absolute out dir : {}".format(abs_out_dir))

            todo = len(os.listdir(abs_batch_dir))
            processed = len(os.listdir(os.path.join(abs_out_dir, 'processed')))
            rejected = len(os.listdir(os.path.join(abs_out_dir, 'rejected')))
            failed = len(os.listdir(os.path.join(abs_out_dir, 'failed')))

            # Get total size of folders
            size_todo = sizeof_fmt(get_size(abs_batch_dir))
            size_processed = sizeof_fmt(get_size(os.path.join(abs_out_dir, 'processed')))
            size_rejected = sizeof_fmt(get_size(os.path.join(abs_out_dir, 'rejected')))
            size_failed = sizeof_fmt(get_size(os.path.join(abs_out_dir, 'failed')))

            os.system("date")
            print(f"todo = {todo} ({size_todo}) / processed = {processed} ({size_processed}) "
                  f"/ rejected = {rejected} ({size_rejected}) / failed = {failed} ({size_failed})")
            print()

    def copy_batch_to_ingest_area(self, source, target):
        """Copies a batch from source to target. Source must be an existing batch directory, target must be a location
        inside one of the inboxes of the ingest area. If target is a directory with the same name as source,
        the contents of source will be copied. If target is a directory with a different name, the contents of source
        will be copied into a new directory with the name of source in target.

        After copying, the mode of the copied files and directories will be set to the value configured in the ingest
        area."""

        abs_source = os.path.abspath(source)
        abs_target = os.path.abspath(target)
        if not os.path.isdir(abs_source):
            print("ERROR: source {source} is not a directory")
            return 1
        if not os.path.isdir(abs_target):
            print("ERROR: target {target} is not a directory")
            return 1

        ingest_area = next(filter(lambda ia: is_sub_path_of(abs_target, ia['inbox']), self.ingest_areas), None)
        if ingest_area is None:
            print(
                f"ERROR: target {target} does not seem to be in one of the inboxes of the ingest areas: {ingest_area}")
            return 1
        else:
            logging.debug(f"Found ingest_area: {ingest_area}")

        # Check if target is a directory with the same name as source
        if os.path.basename(abs_source) == os.path.basename(abs_target):
            target_dir = abs_target
        else:
            target_dir = os.path.join(abs_target, os.path.basename(abs_source))

        if os.path.exists(target_dir):
            for f in os.listdir(abs_source):
                shutil.copy(os.path.join(abs_source, f), target_dir)
        else:
            shutil.copytree(abs_source, target_dir)

        print("Setting mode and group of copied files and directories...")
        set_permissions(target_dir, file_mode=ingest_area['deposits_mode']['file'],
                        dir_mode=ingest_area['deposits_mode']['directory'], group=ingest_area['deposits_group'])



import argparse
import os

from datastation.common.batch_processing import BatchProcessor, get_pids
from datastation.common.config import init
from datastation.common.utils import add_batch_proccessor_args, add_dry_run_arg
from datastation.dataverse.dataverse_client import DataverseClient

exporter_to_extension = {
    'dcterms': 'xml',
    'ddi': 'xml',
    'Datacite': 'xml',
    'html': 'html',
    'dataverse_json': 'json',
    'oai_dc': 'xml',
    'OAI_ORE': 'xml',
    'oai_datacite': 'xml',
    'schema.org': 'json'
}


def get_metadata_export(args, pid, dataverse):
    result = dataverse.dataset(pid).get_metadata_export(dry_run=args.dry_run, exporter=args.exporter)
    if args.output_dir:
        if not os.path.exists(args.output_dir):
            os.makedirs(args.output_dir)
        pid = pid.replace('/', '_')
        with open(f'{args.output_dir}/{pid}.{exporter_to_extension[args.exporter]}', 'w') as f:
            f.write(result)
    else:
        print(result)


def main():
    config = init()
    dataverse = DataverseClient(config['dataverse'])

    parser = argparse.ArgumentParser(description='Get metadata export for a dataset. Note that Dataverse currently '
                                                 'only supports getting metadata exports for the latest published '
                                                 'version of a dataset.')

    parser.add_argument('pid_or_pids_file',
                        help='the pid of the dataset to get the metadata export for, or a file with a list of pids')
    parser.add_argument('-e', '--exporter', default='dataverse_json',
                        help=f"the exporter to use (one of: {', '.join(exporter_to_extension.keys())}; "
                             f"default: dataverse_json)",
                        dest='exporter')
    parser.add_argument('-o', '--output-dir', help="the output directory where the exported metadata files will be "
                                                   "stored. If not provided, the files will be dumped to stdout. If "
                                                   "the directory does not exist, it will be created.",
                        dest='output_dir')
    add_batch_proccessor_args(parser)
    add_dry_run_arg(parser)

    args = parser.parse_args()
    batch_processor = BatchProcessor(wait=args.wait, fail_on_first_error=args.fail_fast)
    pids = get_pids(args.pid_or_pids_file)
    batch_processor.process_pids(pids,
                                 callback=lambda pid: get_metadata_export(args, pid, dataverse))


if __name__ == '__main__':
    main()



import json
import logging
import sys

from datastation.common.result_writer import JsonResultWriter
from datastation.dataverse.dataverse_client import DataverseClient


class Datasets:
    def __init__(self, dataverse_client: DataverseClient, dry_run: bool = False):
        self.dataverse_client = dataverse_client
        self.dry_run = dry_run

    def print_dataset_attributes(self, storage: bool, role: str, pid: str):
        def process_dataset_attributes(
                dataset: dict,
                storage: bool,
                role: str,
                result_writer: JsonResultWriter = None,
                is_first: bool = False
        ):
            # convert search result to dataset if needed
            if "global_id" in dataset:
                result = self.dataverse_client.dataset(dataset["global_id"]).get(
                    dry_run=self.dry_run
                )

                if result is None:
                    raise Exception("Dataset not found")

                dataset = result

            pid = dataset["datasetPersistentId"]
            attributes = {"pid": pid}

            if storage:
                if dataset:
                    attributes["storage"] = sum(
                        f["dataFile"]["filesize"] for f in dataset["files"]
                    )
                else:
                    raise Exception("Dataset not found")

            if role is not None:
                role_assignments = self.dataverse_client.dataset(
                    pid
                ).get_role_assignments(dry_run=self.dry_run)

                if role_assignments is not None:
                    attributes["users"] = [
                        user["assignee"].replace("@", "")
                        for user in role_assignments
                        if user["_roleAlias"] == role
                    ]

            if result_writer:
                result_writer.write(attributes, is_first)
            else:
                print(json.dumps(attributes, indent=2))

        # a single dataset
        if pid is not None:
            dataset = self.dataverse_client.dataset(pid).get(dry_run=self.dry_run)

            if dataset:
                process_dataset_attributes(dataset, storage, role)
            else:
                raise Exception("Dataset not found")

            return

        else:
            # all datasets
            contents = self.dataverse_client.dataverse().search(dry_run=self.dry_run)

            result_writer = JsonResultWriter(out_stream=sys.stdout)
            first = True

            try:
                for result in contents:
                    try:
                        process_dataset_attributes(result, storage, role, result_writer, is_first=first)
                        first = False
                    except Exception as e:
                        logging.error(f"Error processing dataset {result['global_id']}: {e}")
                        continue
            finally:
                result_writer.close()



import argparse

from datastation.common.batch_processing import get_pids, BatchProcessorWithReport
from datastation.common.config import init
from datastation.common.utils import add_batch_proccessor_args, add_dry_run_arg
from datastation.dataverse.dataverse_client import DataverseClient
from datastation.dataverse.destroy_placeholder_dataset import destroy_placeholder_dataset


def main():
    config = init()
    dataverse = DataverseClient(config['dataverse'])

    parser = argparse.ArgumentParser(
        description='Destroys a dataset that is a placeholder for a dataset that has not yet been migrated. In order '
                    'to validate that the dataset is a placeholder, the dataset must be published. Furthermore, '
                    'the description of the dataset must match the pattern configured in '
                    'migration_placeholders.description_text_pattern. Note, that the safety latch must also be OFF.')
    parser.add_argument('pid_or_pids_file', help='The pid of the dataset to destroy, or a file with a list of pids')
    add_batch_proccessor_args(parser)
    add_dry_run_arg(parser)
    args = parser.parse_args()

    batch_processor = BatchProcessorWithReport(wait=args.wait, report_file=args.report_file,
                                               headers=['PID', 'Destroyed', 'Messages'])
    pids = get_pids(args.pid_or_pids_file)
    description_text_pattern = config['migration_placeholders']['description_text_pattern']
    batch_processor.process_pids(pids,
                                 callback=lambda pid, csv_report: destroy_placeholder_dataset(dataverse.dataset(pid),
                                                                                              description_text_pattern,
                                                                                              csv_report))


if __name__ == '__main__':
    main()



import json
import time

import requests

from datastation.common.utils import print_dry_run_message


class DatasetApi:

    def __init__(self, pid, server_url, api_token, unblock_key, safety_latch):
        self.pid = pid
        self.server_url = server_url
        self.api_token = api_token
        self.unblock_key = unblock_key
        self.safety_latch = safety_latch

    def get_pid(self):
        return self.pid
    
    def get(self, version=":latest", dry_run=False):
        url = f'{self.server_url}/api/datasets/:persistentId/versions/{version}'
        headers = {'X-Dataverse-key': self.api_token}
        params = {'persistentId': self.pid}
        
        if dry_run:
            print_dry_run_message(method='GET', url=url, headers=headers, params=params)
            return None
        
        dv_resp = requests.get(url, headers=headers, params=params)
        dv_resp.raise_for_status()

        resp_data = dv_resp.json()['data']
        return resp_data

    def get_role_assignments(self, dry_run=False):
        url = f'{self.server_url}/api/datasets/:persistentId/assignments'
        params = {'persistentId': self.pid}
        headers = {'X-Dataverse-key': self.api_token}
        if dry_run:
            print_dry_run_message(method='GET', url=url, headers=headers, params=params)
            return None
        else:
            r = requests.get(url, headers=headers, params=params)
        r.raise_for_status()
        return r.json()['data']

    def add_role_assignment(self, assignee, role, dry_run=False):
        url = f'{self.server_url}/api/datasets/:persistentId/assignments'
        params = {'persistentId': self.pid}
        headers = {'X-Dataverse-key': self.api_token, 'Content-type': 'application/json'}
        role_assignment = {"assignee": assignee, "role": role}
        if dry_run:
            print_dry_run_message(method='POST', url=url, headers=headers, params=params,
                                  data=json.dumps(role_assignment))
            return None
        else:
            r = requests.post(url, headers=headers, params=params, json=role_assignment)
            r.raise_for_status()
            return r

    def remove_role_assignment(self, assignment_id, dry_run=False):
        url = f'{self.server_url}/api/datasets/:persistentId/assignments/{assignment_id}'
        params = {'persistentId': self.pid}
        headers = {'X-Dataverse-key': self.api_token, 'Content-type': 'application/json'}
        if dry_run:
            print_dry_run_message(method='DELETE', url=url, headers=headers, params=params)
            return None
        else:
            r = requests.delete(url, headers=headers, params=params)
        r.raise_for_status()
        return r

    def is_draft(self, dry_run=False):
        url = f'{self.server_url}/api/datasets/:persistentId'
        params = {'persistentId': self.pid}
        headers = {'X-Dataverse-key': self.api_token}
        if dry_run:
            print_dry_run_message(method='GET', url=url, headers=headers, params=params)
            return None
        else:
            r = requests.get(url, headers=headers, params=params)
        r.raise_for_status()
        return r.json()['data']['latestVersion']['versionState'] == 'DRAFT'

    def delete_draft(self, dry_run=False):
        url = f'{self.server_url}/api/datasets/:persistentId'
        params = {'persistentId': self.pid}
        headers = {'X-Dataverse-key': self.api_token}
        if dry_run:
            print_dry_run_message(method='DELETE', url=url, headers=headers, params=params)
            return None
        else:
            r = requests.delete(url, headers=headers, params=params)
        r.raise_for_status()
        return r.json()

    def destroy(self, dry_run=False):
        url = f'{self.server_url}/api/datasets/:persistentId/destroy'
        headers = {'X-Dataverse-key': self.api_token}
        params = {'persistentId': self.pid, 'unblock-key': self.unblock_key}

        if self.safety_latch:
            print("Safety latch is on, not sending command...")
            return None
        else:
            if dry_run:
                print_dry_run_message(method='DELETE', url=url, headers=headers, params=params)
                return None
            r = requests.delete(url, headers=headers, params=params)
        r.raise_for_status()
        return r.json()

    def get_metadata(self, version=':latest', dry_run=False):
        """Get the native JSON metadata for a dataset version. Version can be a number of one of ':latest', ':draft' or
         ':latest-published'. See
         https://guides.dataverse.org/en/latest/api/native-api.html#get-json-representation-of-a-dataset
         for more information.
         """
        url = f'{self.server_url}/api/datasets/:persistentId/versions/{version}'
        params = {'persistentId': self.pid}
        headers = {'X-Dataverse-key': self.api_token}
        if dry_run:
            print_dry_run_message(method='GET', url=url, headers=headers, params=params)
            return None
        else:
            r = requests.get(url, headers=headers, params=params)
        r.raise_for_status()
        return r.json()['data']

    def get_metadata_export(self, exporter='dataverse_json', dry_run=False):
        """Get a metadata export for the latest version of the dataset.
        See https://guides.dataverse.org/en/latest/api/native-api.html#export-metadata-of-a-dataset-in-various-formats
         for more information."""
        # N.B. no :persistentId in the URL
        url = f'{self.server_url}/api/datasets/export?exporter={exporter}'
        params = {'persistentId': self.pid}
        headers = {'X-Dataverse-key': self.api_token}
        if dry_run:
            print_dry_run_message(method='GET', url=url, headers=headers, params=params)
            return None
        else:
            r = requests.get(url, headers=headers, params=params)
        r.raise_for_status()
        return r.text

    def get_locks(self, lock_type=None, dry_run=False):
        url = f'{self.server_url}/api/datasets/:persistentId/locks'
        params = {'persistentId': self.pid}
        if lock_type:
            params['type'] = lock_type
        headers = {'X-Dataverse-key': self.api_token}
        if dry_run:
            print_dry_run_message(method='GET', url=url, headers=headers, params=params)
            return None
        else:
            r = requests.get(url, headers=headers, params=params)
        r.raise_for_status()
        return r.json()['data']

    def add_lock(self, lock_type, dry_run=False):
        url = f'{self.server_url}/api/datasets/:persistentId/lock/{lock_type}'
        params = {'persistentId': self.pid}
        headers = {'X-Dataverse-key': self.api_token, 'Content-type': 'application/json'}
        if dry_run:
            print_dry_run_message(method='POST', url=url, headers=headers, params=params)
            return None
        else:
            r = requests.post(url, headers=headers, params=params)
            r.raise_for_status()
            return r.json()

    def remove_lock(self, lock_type=None, dry_run=False):
        url = f'{self.server_url}/api/datasets/:persistentId/locks'
        params = {'persistentId': self.pid}
        if lock_type:
            params['type'] = lock_type
        headers = {'X-Dataverse-key': self.api_token}
        if dry_run:
            print_dry_run_message(method='DELETE', url=url, headers=headers, params=params)
            return None
        else:
            r = requests.delete(url, headers=headers, params=params)
        r.raise_for_status()
        return r.json()

    def remove_all_locks(self, dry_run=False):
        return self.remove_lock(dry_run=dry_run)

    def publish(self, update_type='major', dry_run=False):
        url = f'{self.server_url}/api/datasets/:persistentId/actions/:publish'
        params = {'persistentId': self.pid, 'type': update_type}
        headers = {'X-Dataverse-key': self.api_token}
        if dry_run:
            print_dry_run_message(method='POST', url=url, headers=headers, params=params)
            return None
        r = requests.post(url, headers=headers, params=params)
        r.raise_for_status()
        return r.json()

    def reindex(self, dry_run=False):
        url = f'{self.server_url}/api/admin/index/dataset'
        params = {'persistentId': self.pid}
        headers = {'X-Dataverse-key': self.api_token}
        if self.unblock_key:
            params['unblock-key'] = self.unblock_key
        if dry_run:
            print_dry_run_message(method='GET', url=url, headers=headers, params=params)
            return None
        else:
            r = requests.get(url, headers=headers, params=params)
        r.raise_for_status()
        return r.json()

    def modify_registration_metadata(self, dry_run=False):
        url = f'{self.server_url}/api/datasets/:persistentId/modifyRegistrationMetadata'
        params = {'persistentId': self.pid}
        headers = {'X-Dataverse-key': self.api_token}
        if dry_run:
            print_dry_run_message(method='POST', url=url, headers=headers, params=params)
            return None
        else:
            r = requests.post(url, headers=headers, params=params)
        r.raise_for_status()
        return r.json()

    def get_files(self, version=':latest', dry_run=False):
        url = f'{self.server_url}/api/datasets/:persistentId/versions/{version}/files'
        params = {'persistentId': self.pid}
        headers = {'X-Dataverse-key': self.api_token}
        if dry_run:
            print_dry_run_message(method='GET', url=url, headers=headers, params=params)
            return None
        else:
            r = requests.get(url, headers=headers, params=params)
        r.raise_for_status()
        return r.json()['data']

    def await_unlock(self, lock_type=None, sleep_time=5, max_tries=10):
        """Wait for a lock to be removed from the dataset."""
        tries = 0
        while tries < max_tries:
            locks = self.get_locks(lock_type=lock_type)
            if len(locks) == 0:
                return
            tries += 1
            time.sleep(sleep_time)
        if lock_type is None:
            message = f'Locks not removed after {max_tries} tries.'
        else:
            message = f'Locks {lock_type} not removed after {max_tries} tries.'
        raise RuntimeError(message)



import logging
from re import match

from datastation.common.csv import CsvReport
from datastation.dataverse.dataset_api import DatasetApi


def description_object_matches(description_text_pattern):
    def matches(description):
        value = description['dsDescriptionValue']['value']
        return match(description_text_pattern, value)

    return matches


def has_directory_label_different_from(file_metadata, dir_label):
    return 'directoryLabel' not in file_metadata or file_metadata['directoryLabel'] != dir_label


def destroy_placeholder_dataset(dataset_api: DatasetApi, description_text_pattern, csv_report: CsvReport,
                                dry_run: bool = False):
    logging.debug("Entering destroy_placeholder_dataset")
    blocker = False
    messages = []
    try:
        dataset_metadata = dataset_api.get_metadata()
        ds_description = next(
            filter(lambda m: m['typeName'] == 'dsDescription',
                   dataset_metadata['metadataBlocks']['citation']['fields']))
        descriptions = ds_description['value']
        logging.debug(f"Descriptions: {descriptions}")
        if len(list(filter(description_object_matches(description_text_pattern), descriptions))) == 0:
            blocker = True
            messages.append(f"No description found matching configured pattern: BLOCKER")
        else:
            messages.append("Description with text pattern found: OK")

        files = dataset_metadata['files']
        logging.debug(f"Files: {files}")

        if len(files) > 4:
            blocker = True
            messages.append(f"More than 4 files found: {len(files)}: BLOCKER")
        else:
            messages.append(f"Found {len(files)} files <= 4: OK")

        non_easy_migration_files = list(
            filter(lambda m: has_directory_label_different_from(m, 'easy-migration'), files))
        logging.debug(f"Non easy-migration files: {non_easy_migration_files}")

        if len(non_easy_migration_files) > 0:
            blocker = True
            messages.append(f"Files other than 'easy-migration' found: {len(non_easy_migration_files)}: BLOCKER")
        else:
            messages.append("Only found easy-migration files: OK")

    except Exception as e:
        blocker = True
        messages.append(f"Could not perform checks on dataset {dataset_api.get_pid()}: BLOCKER")

    if blocker:
        csv_report.write({'PID': dataset_api.get_pid(), 'Destroyed': False, 'Messages': '; '.join(messages)})
        logging.warning(f"BLOCKERS FOUND, NOT PERFORMING DESTROY FOR {dataset_api.get_pid()}")
    else:
        dataset_api.destroy(dry_run=dry_run)
        csv_report.write({'PID': dataset_api.get_pid(), 'Destroyed': True, 'Messages': '; '.join(messages)})



import argparse
from typing import List, Optional

from datastation.common.config import init
from datastation.common.utils import add_dry_run_arg
from datastation.dataverse.datasets import Datasets
from datastation.dataverse.dataverse_client import DataverseClient


class AttributeOptions:
    user_role: Optional[str] = None
    storage: bool = False


def main():
    config = init()
    parser = argparse.ArgumentParser(description="Retrieves attributes of a dataset")

    parser.add_argument(
        "--user-with-role",
        dest="user_with_role",
        help="List users with a specific role on the dataset",
    )
    parser.add_argument(
        "--storage",
        dest="storage",
        action="store_true",
        help="The storage in bytes",
    )

    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument("pid", help="The dataset pid", nargs="?")
    group.add_argument(
        "--all",
        dest="all_datasets",
        action="store_true",
        help="All datasets in the dataverse",
        required=False,
    )

    add_dry_run_arg(parser)
    args = parser.parse_args()

    dataverse_client = DataverseClient(config["dataverse"])

    datasets = Datasets(dataverse_client, dry_run=args.dry_run)
    datasets.print_dataset_attributes(args.storage, args.user_with_role, args.pid)


if __name__ == "__main__":
    main()



from typing import Union
import psycopg
from psycopg.rows import TupleRow


class Database:
    def __init__(self, config):
        self.host = config["host"]
        self.dbname = config["dbname"]
        self.user = config["user"]
        self.password = config["password"]
        self.connection: psycopg.Connection[TupleRow] | None = None

    def connect(self):
        self.connection = psycopg.connect(
            f"host={self.host} dbname={self.dbname} user={self.user} password={self.password}"
        )

    def query(self, query):
        if self.connection is None:
            raise Exception("No connection to database")

        with self.connection.cursor() as cursor:
            cursor.execute(query)
            return cursor.fetchall()

    def update(self, query, *args):
        if self.connection is None:
            raise Exception("No connection to database")

        with self.connection.cursor() as cursor:
            result = cursor.execute(query, *args)
            self.connection.commit()
            return result.rowcount

    def close(self):
        if self.connection is not None:
            self.connection.close()

    def __enter__(self):
        self.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()



import os


def find_bags(path, max_depth=1) -> list:
    """
    Find all bags in the given path, up to the given depth. If the path is a bag, it will be returned. If the path is
    a directory, it will be searched for bags. If the path is a file, it will be ignored.

    Note that, for the purposes of this function, a bag is defined as a directory that contains a file named bagit.txt.
    The function does not check the contents of the file, nor does it check that the directory contains any other
    files or directories required to make it a valid bag.

    :param path: The path to search for bags.
    :param max_depth: The maximum depth to search for bags. 0 means only
      the given path will be examined to see if the directory is bag, 1 means the given path and its immediate children
      will be searched, 2 will include grandchildren, etc. A negative value means the search will be unlimited.
    """
    if os.path.isfile(path):
        return
    if is_bag(path):
        yield path
    if max_depth == 0:
        return
    for entry in os.listdir(path):
        entry_path = os.path.join(path, entry)
        if os.path.isdir(entry_path):
            yield from find_bags(entry_path, max_depth - 1)


def is_bag(path: str) -> bool:
    return os.path.exists(os.path.join(path, 'bagit.txt'))



import argparse
from datetime import datetime

import rich

from datastation.common.batch_processing import get_pids, BatchProcessorWithReport
from datastation.common.config import init
from datastation.common.utils import add_batch_proccessor_args, add_dry_run_arg
from datastation.dataverse.dataset_api import DatasetApi
from datastation.dataverse.dataverse_client import DataverseClient


def add_role_assignments(args, dataverse_client: DataverseClient):
    pids = get_pids(args.pid_or_pid_file)
    batch_processor = BatchProcessorWithReport(wait=args.wait, fail_on_first_error=args.fail_fast,
                                               report_file=args.report_file,
                                               headers=['DOI', 'Modified', 'Assignee', 'Role', 'Change'])
    batch_processor.process_pids(pids,
                                 lambda pid, csv_report: add_role_assignment(args.role_assignment,
                                                                             dataset_api=dataverse_client.dataset(pid),
                                                                             csv_report=csv_report))


def add_role_assignment(role_assignment, dataset_api: DatasetApi, csv_report, dry_run: bool = False):
    assignee = role_assignment.split('=')[0]
    role = role_assignment.split('=')[1]
    action = "None"
    if in_current_assignments(assignee, role, dataset_api):
        print("{} is already {} for dataset {}".format(assignee, role, dataset_api.get_pid()))
    else:
        print(
            "Adding {} as {} for dataset {}".format(assignee, role, dataset_api.get_pid()))
        dataset_api.add_role_assignment(assignee, role, dry_run=dry_run)
        action = "Added"
    csv_report.write(
        {'DOI': dataset_api.get_pid(), 'Modified': datetime.now(), 'Assignee': assignee, 'Role': role,
         'Change': action})


def in_current_assignments(assignee, role, dataset_api: DatasetApi):
    current_assignments = dataset_api.get_role_assignments()
    found = False
    for current_assignment in current_assignments:
        if current_assignment.get('assignee') == assignee and current_assignment.get(
                '_roleAlias') == role:
            found = True
            break
    return found


def list_role_assignments(args, dataverse_client):
    r = dataverse_client.dataset(args.pid).get_role_assignments()
    if r is not None:
        rich.print_json(data=r)


def remove_role_assignments(args, dataverse_client: DataverseClient):
    pids = get_pids(args.pid_or_pid_file)
    batch_processor = BatchProcessorWithReport(wait=args.wait, report_file=args.report_file,
                                               headers=['DOI', 'Modified', 'Assignee', 'Role', 'Change'])
    batch_processor.process_pids(pids,
                                 lambda pid, csv_report: remove_role_assignment(args.role_assignment,
                                                                                dataset_api=dataverse_client.dataset(
                                                                                    pid),
                                                                                csv_report=csv_report))


def remove_role_assignment(role_assignment, dataset_api: DatasetApi, csv_report, dry_run: bool = False):
    assignee = role_assignment.split('=')[0]
    role = role_assignment.split('=')[1]
    action = "None"
    if in_current_assignments(assignee, role, dataset_api):
        print("Removing {} as {} for dataset {}".format(assignee, role, dataset_api.get_pid()))
        all_assignments = dataset_api.get_role_assignments()
        for assignment in all_assignments:
            if assignment.get('assignee') == assignee and assignment.get('_roleAlias') == role:
                dataset_api.remove_role_assignment(assignment.get('id'), dry_run=dry_run)
                action = "Removed"
                break
    else:
        print("{} is not {} for dataset {}".format(assignee, role, dataset_api.get_pid()))
    csv_report.write(
        {'DOI': dataset_api.get_pid(), 'Modified': datetime.now(), 'Assignee': assignee, 'Role': role,
         'Change': action})


def main():
    config = init()
    dataverse_client = DataverseClient(config['dataverse'])
    batch_processor = BatchProcessorWithReport(headers=['DOI', 'Modified', 'Assignee', 'Role', 'Change'])

    # Create main parser and subparsers
    parser = argparse.ArgumentParser(description='Manage role assignments on one or more datasets.')
    subparsers = parser.add_subparsers(help='subcommands', dest='subcommand')

    # Add role assignment
    parser_add = subparsers.add_parser('add', help='add role assignment to specified dataset(s)')
    parser_add.add_argument('role_assignment',
                            help='role assignee and alias (example: @dataverseAdmin=contributor) to add')
    parser_add.add_argument('pid_or_pid_file', help='the dataset pid or the input file with the dataset pids')
    add_batch_proccessor_args(parser_add)
    add_dry_run_arg(parser_add)

    parser_add.set_defaults(func=lambda _: add_role_assignments(_, dataverse_client))

    # Remove role assignment
    parser_remove = subparsers.add_parser('remove', help='remove role assignment from specified dataset(s)')
    parser_remove.add_argument('role-assignment',
                               help='role assignee and alias (example: @dataverseAdmin=contributor)')
    parser_remove.add_argument('pid_or_pid_file', help='The dataset pid or the input file with the dataset pids')
    add_batch_proccessor_args(parser_remove)
    add_dry_run_arg(parser_remove)
    parser_remove.set_defaults(func=lambda _: remove_role_assignments(_, dataverse_client))

    # List role assignments
    parser_list = subparsers.add_parser('list',
                                        help='list role assignments for specified dataset (only one pid allowed)')
    parser_list.add_argument('pid', help='the dataset pid')
    add_dry_run_arg(parser_list)
    parser_list.set_defaults(func=lambda _: list_role_assignments(_, dataverse_client))

    args = parser.parse_args()
    args.func(args)


if __name__ == '__main__':
    main()



# module for result writer classes
import csv
import json
import typing

import yaml


# Abstract base class for the result writers
class ResultWriter:
    def write(self, result: dict, is_first: bool):
        raise NotImplementedError()

    def close(self):
        raise NotImplementedError()


# Function that write a result in the form of a dictionary to a text stream
class JsonResultWriter(ResultWriter):
    def __init__(self, out_stream: typing.TextIO):
        self.first_result_written = False
        self.out_stream = out_stream

    def write(self, result: dict, is_first: bool):
        if is_first:
            self.out_stream.write("[")
            self.first_result_written = True
        else:
            self.out_stream.write(", ")
        self.out_stream.write(json.dumps(result))
        self.out_stream.flush()

    def close(self):
        if not self.first_result_written:
            self.out_stream.write("[")
        self.out_stream.write("]")


class YamlResultWriter(ResultWriter):
    def __init__(self, out_stream: typing.TextIO):
        self.out_stream = out_stream

    def write(self, result: dict, is_first: bool):
        self.out_stream.write(yaml.dump(result))

    def close(self):
        pass


class CsvResultWriter(ResultWriter):
    def __init__(self, headers: list, out_stream: typing.TextIO):
        self.out_stream = out_stream
        self.headers = headers
        self.csv_writer = csv.DictWriter(out_stream, fieldnames=headers, lineterminator="\n")
        self.csv_writer.writeheader()

    def write(self, result: dict, is_first: bool):
        if len(result.keys()) > 0:
            if set(result.keys()) != set(self.headers):
                raise ValueError("Result keys do not match headers")
            else:
                self.csv_writer.writerow(result)

    def close(self):
        pass



from datastation.common.csv import CsvInput
from datastation.common.database import Database
from datastation.dataverse.builtin_users import User
from datastation.dataverse.dataverse_client import DataverseClient


def update_password(database, user, dry_run=False):
    update_statement = f"UPDATE builtinuser SET encryptedpassword = '{user.encrypted_password}' WHERE username = '{user.username}';"
    if dry_run:
        print(f"dry-run, not updating database with {update_statement}")
    else:
        database.update(update_statement)


class UserImport:
    """Imports users from a CSV file into a Dataverse instance. The CSV file can be exported from EASY or manually
    created. The Dataverse instance must have the BuiltinUsers.KEY set. """

    def __init__(self, dataverse_client: DataverseClient, is_easy_format: bool,
                 builtin_users_key: str, dry_run: bool = False):
        self.dataverse_client = dataverse_client
        self.is_easy_format = is_easy_format
        self.builtin_users_key = builtin_users_key
        self.dry_run = dry_run

    def import_users(self, csv_file):
        with Database(self.dataverse_client.db_config) as database:
            with UserCsv(csv_file, is_easy_format=self.is_easy_format) as userCsv:
                for user in userCsv:
                    self.import_user(user, self.builtin_users_key, database)

    def import_user(self, user, builtin_users_key, database, dry_run=False):
        r = self.dataverse_client.built_in_users(builtin_users_key=builtin_users_key).create(user,
                                                                                             send_email_notification=False,
                                                                                             dry_run=self.dry_run)
        if r is None:
            return
        if r.status_code == 200:
            update_password(database, user, dry_run)
            print(f"Imported user {user.username}")
        else:
            print(f"Error creating user {user}: {r.status_code} {r.json()['message']}")


class UserCsv(CsvInput):

    def __init__(self, csv_file, is_easy_format):
        super().__init__(csv_file, delimiter=";" if is_easy_format else ",")
        self.is_easy_format = is_easy_format

    def __iter__(self):
        for row in super().__iter__():
            if self.is_easy_format:
                last_name = (row["PREFIX"], row["SURNAME"])
                # the PASSWORD-HASH starts with '{SHA}' which needs to be stripped
                encrypted_password = row["PASSWORD-HASH"][5:]
                yield User(row["UID"], row["INITIALS"], " ".join(last_name), row["EMAIL"], row["ORGANISATION"],
                           row["FUNCTION"], encrypted_password)
            else:
                yield User(row["Username"], row["GivenName"], row["FamilyName"], row["Email"], row["Affiliation"],
                           row["Position"], row["encryptedpassword"])



import json

import requests


class BannerApi:

    def __init__(self, server_url: str, api_token: str, unblock_key: str):
        self.server_url = server_url
        self.api_token = api_token
        self.unblock_key = unblock_key

    def list(self, dry_run: bool = False):
        """ List all banners. """
        url = f'{self.server_url}/api/admin/bannerMessage'
        headers = {'X-Dataverse-key': self.api_token}
        if dry_run:
            print(f"Would have sent the following request: {url}")
            return
        r = requests.get(url, headers=headers, params={'unblock-key': self.unblock_key})
        r.raise_for_status()
        return r

    def add(self, msg: str, dismissible_by_user: bool = False, lang: str = 'en', dry_run: bool = False):
        """ Add a banner. """
        banner = {
            "messageTexts": [
                {
                    "lang": lang,
                    "message": msg
                }
            ],
            "dismissibleByUser": str(dismissible_by_user).lower()
        }
        url = f'{self.server_url}/api/admin/bannerMessage'
        headers = {'X-Dataverse-key': self.api_token, 'Content-type': 'application/json'}
        if dry_run:
            print(f"Would have sent the following request: {url}")
            print(json.dumps(banner, indent=4))
            return
        r = requests.post(url, headers=headers, params={'unblock-key': self.unblock_key}, json=banner)
        r.raise_for_status()
        return r

    def remove(self, banner_id: int, dry_run: bool = False):
        """ Remove a banner. """
        url = f'{self.server_url}/api/admin/bannerMessage/{banner_id}'
        headers = {'X-Dataverse-key': self.api_token}
        if dry_run:
            print(f"Would have sent the following request: {url}")
            return
        r = requests.delete(url, headers=headers, params={'unblock-key': self.unblock_key})
        r.raise_for_status()
        return r



import json as jsonlib
import os
import shutil
import argparse


def add_dry_run_arg(parser):
    parser.add_argument('-d', '--dry-run', action='store_true',
                        help='Do not perform the action, but show what would be done.')


def add_batch_proccessor_args(parser, report: bool = True):
    parser.add_argument('-w', '--wait-between-items', default=2.0, type=float,
                        help="number of seconds to wait between processing items",
                        dest='wait')
    parser.add_argument('-f', '--fail-fast', dest='fail_fast', action='store_true', help='fail on first error ')
    if report:
        parser.add_argument('-r', '--report-file', default='-', help="the report file, or - for stdout",
                            dest='report_file')

def positive_int_argument_converter(value):
    try:
        ivalue = int(value)

        if ivalue <= 0:
            raise argparse.ArgumentTypeError("value must be a number greater than zero")

        return ivalue
    except ValueError:
        raise argparse.ArgumentTypeError("value must be a number greater than zero")


def print_dry_run_message(method, url, params=None, headers=None, data=None, json=None):
    print("DRY-RUN: only printing command, not sending it...")
    print(f"{method} {url}")
    print(f"headers: {headers}") if headers else None
    print(f"params: {params}") if params else None
    print(f"data: {data}") if data else None
    print(f"json: {jsonlib.dumps(json)}") if json else None
    print()


def is_sub_path_of(child, parent):
    """
    Returns True if child is a descendant directory of parent, or if child is a file in a descendant directory
    of parent. Returns False otherwise. If one of the paths does not exist, returns False. The paths are first
    converted to absolute paths."""
    if not os.path.exists(child) or not os.path.exists(parent):
        return False
    absolute_parent = os.path.abspath(parent)
    absolute_dir = os.path.abspath(child)
    return absolute_dir.startswith(absolute_parent)


def has_file_pred(file, pred):
    return pred(file)


def has_dirtree_pred(path, pred):
    """
    Returns True if all files and directories in the directory tree rooted at path satisfy the predicate pred.
    """
    for root, dirs, files in os.walk(path):
        return pred(root) \
            and all(pred(os.path.join(root, d)) for d in dirs) \
            and all(pred(os.path.join(root, f)) for f in files)


def set_permissions(path, dir_mode, file_mode, group):
    """
    Sets the permissions of all files and directories in the directory tree rooted at path to dir_mode and file_mode,
    respectively. The group of all files and directories is set to group.
    """
    for root, dirs, files in os.walk(path):
        shutil.chown(root, group=group)
        for d in [root] + dirs:
            p = os.path.join(root, d)
            os.chmod(p, dir_mode)
            shutil.chown(p, group=group)
        for f in files:
            p = os.path.join(root, f)
            os.chmod(p, file_mode)
            shutil.chown(p, group=group)


def get_size(start_path='.'):
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(start_path):
        for f in filenames:
            # skip if it is symbolic link
            if not os.path.islink(os.path.join(dirpath, f)):
                total_size += os.path.getsize(os.path.join(dirpath, f))
    return total_size


# Return size in human-readable format
def sizeof_fmt(num, suffix='B'):
    for unit in ['', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi']:
        if abs(num) < 1024.0:
            return "%3.1f%s%s" % (num, unit, suffix)
        num /= 1024.0
    return "%.1f%s%s" % (num, 'Yi', suffix)



import argparse
import logging

from datastation.common.config import init
from datastation.dataverse.dataverse_client import DataverseClient


def find_datasets_by_role_assignment(database, role_assignment):
    [role_assignee, role_alias] = role_assignment.split('=')
    logging.debug(f"role_assignee={role_assignee}, role_alias={role_alias}")
    select_statement = \
        f"select concat(dvo.protocol, ':', dvo.authority, '/', dvo.identifier) " \
        f"from roleassignment ra inner join dataverserole dr on ra.role_id=dr.id " \
        f"inner join dvobject dvo on definitionpoint_id=dvo.id " \
        f"where dtype='Dataset' and assigneeidentifier='{role_assignee}' and alias='{role_alias}'"
    logging.debug(f"select_statement={select_statement}")
    result = database.query(select_statement)
    if len(result) == 0:
        print(f"No datasets for user {role_assignee} with role {role_alias}")
    for r in result:
        print(r[0])


def main():
    config = init()
    dataverse = DataverseClient(config['dataverse'])
    parser = argparse.ArgumentParser(description='Find datasets by role assignment.')
    parser.add_argument('role_assignment', help='The role assignment to find, e.g. "@user1=curator"')
    args = parser.parse_args()

    with dataverse.database() as database:
        find_datasets_by_role_assignment(database, args.role_assignment)


if __name__ == '__main__':
    main()



import argparse
import json
from datetime import datetime

from datastation.common.batch_processing import BatchProcessorWithReport, get_pids
from datastation.common.config import init
from datastation.common.utils import add_batch_proccessor_args, add_dry_run_arg
from datastation.dataverse.dataverse_client import DataverseClient


def publish_datasets(args, dataverse_client: DataverseClient):
    pids = get_pids(args.pid_or_pid_file)
    batch_processor = BatchProcessorWithReport(report_file=args.report_file, wait=args.wait,
                                               fail_on_first_error=args.fail_fast,
                                               headers=['DOI', 'Modified', 'Change', 'Messages'])
    batch_processor.process_pids(pids,
                                 lambda pid, csv_report: publish(pid, dataverse_client,
                                                                 update_type=args.update_type,
                                                                 csv_report=csv_report,
                                                                 dry_run=args.dry_run))


def publish(pid, dataverse_client: DataverseClient, update_type, csv_report, dry_run: bool = False):
    dataset_api = dataverse_client.dataset(pid)
    message = ''
    if dataset_api.is_draft():
        print("Dataset {} is a draft. Publishing it.".format(pid))
        r = dataset_api.publish(update_type=update_type, dry_run=dry_run)
        action = "Published"
        if r is not None:
            message = json.dumps(r['data'])
    else:
        print(f"Dataset {pid} is not in draft state. Skipping it.")
        action = "None"
    csv_report.write({'DOI': pid, 'Modified': datetime.now(), 'Change': action, 'Messages': message})


def main():
    config = init()
    dataverse = DataverseClient(config['dataverse'])
    update_types = ['major', 'minor', 'updatecurrent']

    parser = argparse.ArgumentParser(description='Publish a dataset.')
    parser.add_argument('pid_or_pid_file',
                        help='the persistent identifier of the dataset or a file containing a list of '
                             'persistent identifiers.')
    parser.add_argument('-u', '--update-type', dest='update_type',
                        help='whether to create a major, version or update the '
                             'current version (default: major)', choices=update_types, default='major')
    add_batch_proccessor_args(parser)
    add_dry_run_arg(parser)

    args = parser.parse_args()
    publish_datasets(args, dataverse)


if __name__ == '__main__':
    main()
